#Requires -version 2.0
#******************************************************************************
#*
#* Module Name: Example.ps1
#*
#* Abstract:    This is a template PowerShell Script file generated by Reflect.
#*              Modify to add your own functionality if required.
#*
#* Returns:     0 - Success.
#*              1 - Error: Invalid XML.
#*              2 - Error: Backup failed.
#*              3 - Error: Not elevated (script will try to execute elevated).
#*
#******************************************************************************
Param([switch]$s, [switch]$full, [switch]$inc, [switch]$diff)
$strReflectPath = "C:\program files\macrium\reflect\Reflect.exe";
$strXmlFilePath = "C:\Users\home\Documents\Reflect\My Backup.xml";
$driveLetterSet = "D:"
$driveLabelSet = "Backup"
$strScriptPath = $MyInvocation.MyCommand.Definition;
#******************************************************************************
#* Func: OnXmlValidationError
#*
#* Desc: Called when a backup fails due to an XML validation error.
#*       This is here to be modified for your own uses.
#*
#******************************************************************************
function OnXmlValidationError()
{
  Write-Warning " ! XML invalid ($strXmlFilePath).";
  # Handle invalid XML error...
   
}
#******************************************************************************
#* Func: OnBackupError
#*
#* Desc: Called when a backup fails.
#*       This is here to be modified for your own uses.
#*
#******************************************************************************
function OnBackupError()
{
  Write-Warning " ! Backup failed ($strXmlFilePath).";
  # Handle backup error...
   
}
#******************************************************************************
#* Func: OnBackupSuccess
#*
#* Desc: Called when a backup succeeds.
#*       This is here to be modified for your own uses.
#*
#******************************************************************************
function OnBackupSuccess()
{
  Write-Host " * Backup succeeded ($strXmlFilePath).";
  # Handle backup success...
   
}
function WaitForHDD()
{
    Register-WmiEvent -Class win32_VolumeChangeEvent -SourceIdentifier volumeChange
    write-host (get-date -format s) "     Beginning script..."
    do{
      $newEvent = Wait-Event -SourceIdentifier volumeChange
      $eventType = $newEvent.SourceEventArgs.NewEvent.EventType
      $eventTypeName = switch($eventType)
        {
        1 {"Configuration changed"}
        2 {"Device arrival"}
        3 {"Device removal"}
        4 {"docking"}
        }
      write-host (get-date -format s) "     Event detected = " $eventTypeName
      if ($eventType -eq 2)
      {
        $driveLetter = $newEvent.SourceEventArgs.NewEvent.DriveName
        $driveLabel = ([wmi]"Win32_LogicalDisk='$driveLetter'").VolumeName
        write-host (get-date -format s) "     Drive name = " $driveLetter
        write-host (get-date -format s) "     Drive label = " $driveLabel
        # Execute process if drive matches specified condition(s)
        if ($driveLetter -eq $driveLetterSet -and $driveLabel -eq $driveLabelSet)
        {
          write-host (get-date -format s) "     Starting task in 3 seconds..."
          start-sleep -seconds 3
          Backup;
        }
      }
      Remove-Event -SourceIdentifier volumeChange
    } while (1-eq1) #Loop until next event
    Unregister-Event -SourceIdentifier volumeChange
  
  Write-Host 'PowerShell script for Macrium Reflect Backup Definition File';
  Write-Host "BDF: $strXmlFilePath";
  Elevate;
  $iExitCode = Backup;
}
#******************************************************************************
#* Func: Main
#*
#* Desc: This is main function to start execution.
#*
#******************************************************************************
function Main()
{
  WaitForHDD;
  Write-Host "Script finished with exit code $iExitCode.";
  Exit $iExitCode;
}
#******************************************************************************
#* Func: Elevate
#*
#* Desc: Elevates this script for UAC.
#*       This means that only one UAC Elevation prompt is displayed and
#*       functions/programs will not fail if they require admin privileges.
#*
#******************************************************************************
function Elevate()
{
  # Only elevate if not ran from the task scheduler.
  Write-Host ' * Checking elevated access rights... ' -NoNewLine;
  if (-Not $s)
  {
    # Check to see if we are currently running "as Administrator"
    if (!([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole]"Administrator"))
    {
      $ElevatedProcess = new-object System.Diagnostics.ProcessStartInfo "PowerShell";
      # Specify the current script path and name as a parameter
      $strType = GetBackupTypeParameter;
      $ElevatedProcess.Arguments = "-ExecutionPolicy Bypass & '" + $script:MyInvocation.MyCommand.Path + "' $strType";
      # Indicate that the process should be elevated
      $ElevatedProcess.Verb = "runas";
      # Start the new process
      [System.Diagnostics.Process]::Start($ElevatedProcess);
      # Exit this unelevated script with exit code for "Error: Not elevated"
      Exit 3;
    }
  }
  Write-Host 'Done.';
}
#******************************************************************************
#* Func: Backup
#*
#* Desc: Calls Reflect.exe passing an XML BDF as a parameter.
#*
#******************************************************************************
function Backup()
{
  Write-Host ' * Running the backup... ' -NoNewLine;
  $strType = GetBackupTypeParameter;
  $strArgs = "-e -w $strType `"$strXmlFilePath`"";
  $iResult = (Start-Process -FilePath $strReflectPath -ArgumentList $strArgs -PassThru -Wait).ExitCode;
  Write-Host 'Done.';
  switch ($iResult)
  {
    2 { OnXmlValidationError; break; }
    1 { OnBackupError;        break; }
    0 { OnBackupSuccess;      break; }
  }
  return $iResult;
}
#******************************************************************************
#* Func: GetBackupTypeParameter
#*
#* Desc: Determines the backup type from command line parameter...
#*        "-full": Full backup
#*        "-inc" : Incremental backup
#*        "-diff": Differential backup
#*
#******************************************************************************
function GetBackupTypeParameter()
{
  if ($full -eq $true) { return '-full'; };
  if ($inc  -eq $true) { return '-inc';  };
  if ($diff -eq $true) { return '-diff'; };
  return ''; # Clone
}
# Execute the Main function
Main;
